
package cz.cuni.mff.corrector.cache;

import cz.cuni.mff.corrector.intent.Intent;
import java.util.Collection;

/**
 * Common interface for cache storages.
 * 
 * @param <Item> Type of object to be stored by the cache manager instance.
 * @author Martin Svoboda
 */
public interface Store<Item> {
    
    // -------------------------------------------------------------------------


    /**
     * Attempts to fetch stored item for a given correction intent.
     * @param indexingIntent Intent to be used as the indexing signature.
     * @return Stored item or <code>null</code> reference.
     */
    public Item getStoredItem(Intent indexingIntent);
    
    /**
     * Inserts a new item into the cache using the intent signature.
     * @param indexingIntent Intent to be used as the indexing signature.
     * @param itemRef Reference to the object to be stored.
     */
    public void setStoredItem(Intent indexingIntent, Item itemRef);
    
    /**
     * Removes the cached item indexed by the given correction intent.
     * @param indexingIntent Intent to be used as the indexing signature.
     */
    public void removeStoredItem(Intent indexingIntent);
    
    /**
     * Returns the list with all stored items of a particular type.
     * @param intentType Specified indexing intent type.
     * @return Independent list with all stored items.
     */
    public Collection<Item> getStoredItems(Intent.Type intentType);
    
    
    // -------------------------------------------------------------------------
    
}
