
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.instruction.InstructionAddLeaf;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Implementation of the add leaf edit operation.
 *
 * @author Martin Svoboda
 */
public class EditAddLeaf extends Edit {

    // -------------------------------------------------------------------------


    /**
     * Reference to the associated repairing instruction.
     */
    private InstructionAddLeaf repairingInstr;


    // -------------------------------------------------------------------------


    /**
     * Creates a new add leaf edit operation object.
     * @param repairingInstr Underlying repairing instruction.
     * @param basePosition Operation base position.
     */
    public EditAddLeaf(
        InstructionAddLeaf repairingInstr,
        Position basePosition
    ) {
        super(basePosition);
        this.repairingInstr = repairingInstr;
    }


    // -------------------------------------------------------------------------


    public Instruction getRepairingInstruction() {
        return this.repairingInstr;
    }

    /**
     * Returns the typed reference to the associated instruction.
     * @return Underlying repairing instruction reference.
     */
    public InstructionAddLeaf getAddLeafInstruction() {
        return this.repairingInstr;
    }

    public String getOperationTitle() {
        return "addLeaf(" +
            this.basePosition.getPositionString() + ", " +
            this.repairingInstr.getNewLabel() + ")";
    }

    
    // -------------------------------------------------------------------------

    
    public EditAddLeaf cloneIdenticalOperation() {
        return new EditAddLeaf(
            this.repairingInstr,
            basePosition.cloneIdenticalPosition()
        );
    }
    
    
    public EditAddLeaf cloneShiftedOperation(int incrementValue) {
        return new EditAddLeaf(
            this.repairingInstr,
            basePosition.cloneShiftedPosition(incrementValue)
        );
    }


    // -------------------------------------------------------------------------

}
