
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.instruction.InstructionRenameLabel;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Implementation of the rename label edit operation.
 *
 * @author Martin Svoboda
 */
public class EditRenameLabel extends Edit {

    // -------------------------------------------------------------------------


    /**
     * Reference to the associated repairing instruction.
     */
    private InstructionRenameLabel repairingInstr;


    // -------------------------------------------------------------------------


    /**
     * Creates a new rename label edit operation object.
     * @param repairingInstr Underlying repairing instruction.
     * @param basePosition Operation base position.
     */
    public EditRenameLabel(
        InstructionRenameLabel repairingInstr,
        Position basePosition
    ) {
        super(basePosition);
        this.repairingInstr = repairingInstr;
    }


    // -------------------------------------------------------------------------


    public Instruction getRepairingInstruction() {
        return this.repairingInstr;
    }

    /**
     * Returns the typed reference to the associated instruction.
     * @return Underlying repairing instruction reference.
     */
    public InstructionRenameLabel getRenameLabelInstruction() {
        return this.repairingInstr;
    }

    public String getOperationTitle() {
        return "renameLabel(" +
            this.basePosition.getPositionString() + ", " +
            this.repairingInstr.getNewLabel() + ")";
    }


    // -------------------------------------------------------------------------


    public EditRenameLabel cloneIdenticalOperation() {
        return new EditRenameLabel(
            this.repairingInstr,
            basePosition.cloneIdenticalPosition()
        );
    }
    
    
    public EditRenameLabel cloneShiftedOperation(int incrementValue) {
        return new EditRenameLabel(
            this.repairingInstr,
            basePosition.cloneShiftedPosition(incrementValue)
        );
    }


    // -------------------------------------------------------------------------

}
