
package cz.cuni.mff.corrector.execution;

import cz.cuni.mff.corrector.corrector.Handler;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

/**
 * Wrapper for the standard executor class used as a pool of workers.
 * 
 * Note that the pool itself does not provide any synchronisation services
 * and just performs the provided requests as they are being provided.
 * 
 * @author Martin Svoboda
 */
public class Pool {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Default number of worker threads in the fixed pool.
     * @see Executors#newFixedThreadPool
     */
    public static int POOL_DEFAULT = 5;
    
    /**
     * Special constant for variable number of worker threads.
     * @see Executors#newCachedThreadPool
     */
    public static Integer POOL_UNLIMITED = null;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Required size of the pool of worker threads.
     * @see #POOL_UNLIMITED
     */
    private Integer poolSize;
    
    
    /**
     * Created and prepared internal pool of workers.
     */
    private Executor threadPool;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Creates and initialises a new pool of workers.
     * @param poolSize Requested size of the pool or <code>null</code>.
     * @see #POOL_UNLIMITED
     */
    public Pool(Integer poolSize) {
        
        // Storing configuration parameters
        this.poolSize = poolSize;
        
        // Initializing an internal pool
        if (this.poolSize == POOL_UNLIMITED) {
            this.threadPool = Executors.newCachedThreadPool();
        } else {
            this.threadPool = Executors.newFixedThreadPool(this.poolSize);
        }
        
    }
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Executes the given prepared correction handler.
     * @param requestedHandler Correction handler instance to be executed.
     */
    public void executeRequestedHandler(Handler requestedHandler) {
        this.threadPool.execute(requestedHandler);
    }
    
    
    // -------------------------------------------------------------------------
    
}
