
package cz.cuni.mff.corrector.instruction;

import cz.cuni.mff.corrector.edit.EditAddLeaf;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Repairing instruction add leaf.
 *
 * @author Martin Svoboda
 */
public class InstructionAddLeaf extends Instruction {

    // -------------------------------------------------------------------------


    /**
     * New label associated with the add leaf instruction.
     */
    private final String newLabel;


    // -------------------------------------------------------------------------


    /**
     * Creates a new instance of an add leaf instruction.
     * @param newLabel New label value.
     */
    public InstructionAddLeaf(String newLabel) {
        this.newLabel = newLabel;
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the associated new label.
     * @return New label associated with the instruction.
     */
    public String getNewLabel() {
        return this.newLabel;
    }


    // -------------------------------------------------------------------------


    public Type getInstructionType() {
        return Type.ADD_LEAF;
    }

    public int getInstructionCost() {
        return COST_UNIT;
    }

    public String getTitleString() {
        return "addLeaf(" + this.newLabel + ")";
    }


    // -------------------------------------------------------------------------


    public EditAddLeaf generateEditOperation() {
        return new EditAddLeaf(this, new Position(0));
    }

    
    // -------------------------------------------------------------------------

}
