
package cz.cuni.mff.corrector.instruction;

import cz.cuni.mff.corrector.edit.EditRenameLabel;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Repairing instruction rename label.
 *
 * @author Martin Svoboda
 */
public class InstructionRenameLabel extends Instruction {

    // -------------------------------------------------------------------------


    /**
     * New label associated with the rename label instruction.
     */
    private final String newLabel;


    // -------------------------------------------------------------------------


    /**
     * Creates a new instance of a rename label instruction.
     * @param newLabel New label value.
     */
    public InstructionRenameLabel(String newLabel) {
        this.newLabel = newLabel;
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the associated new label.
     * @return New label associated with the instruction.
     */
    public String getNewLabel() {
        return this.newLabel;
    }


    // -------------------------------------------------------------------------


    public Type getInstructionType() {
        return Type.RENAME_LABEL;
    }

    public int getInstructionCost() {
        return COST_UNIT;
    }

    public String getTitleString() {
        return "renameLabel(" + this.newLabel + ")";
    }


    // -------------------------------------------------------------------------


    public EditRenameLabel generateEditOperation() {
        return new EditRenameLabel(this, new Position(0));
    }


    // -------------------------------------------------------------------------

}
