
package cz.cuni.mff.corrector.intent;

import cz.cuni.mff.corrector.grammar.Context;
import cz.cuni.mff.corrector.repair.Mend;
import cz.cuni.mff.corrector.tree.Node;
import java.util.EnumSet;
import java.util.List;
import java.util.Set;

/**
 * Implementation of the correct correction intent.
 *
 * @author Martin Svoboda
 */
public class IntentCorrect extends Intent {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * List of all intent types allowed in further nesting.
     */
    private static final Set<Type> ALLOWED_INTENTS =
        EnumSet.of(Type.INSERT, Type.DELETE, Type.REPAIR, Type.RENAME);
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Creates a new instance of the correct correction intent.
     * @param nodeSequence Sequence of nested nodes to be processed.
     * @param grammarContext Nested grammar context reference.
     */
    public IntentCorrect(
        List<Node> nodeSequence,
        Context grammarContext
    ) {
        
        // Constructing core components
        super(
            null,
            null,
            null,
            null,
            nodeSequence,
            grammarContext,
            null
        );
        
    }
    
    
    // -------------------------------------------------------------------------
    
    
    @Override
    public Type getIntentType() {
        return Type.CORRECT;
    }
    
    @Override
    public Set<Type> getAllowedIntents() {
        return ALLOWED_INTENTS;
    }
    
    @Override
    protected String computeIntentSignature() {
        return Type.CORRECT.fullCode;
    }
    
    @Override
    protected String computeIdentifierPostfix() {
        return Type.CORRECT.fullCode;
    }
    
    
    // -------------------------------------------------------------------------


    /**
     * Generates all mends for the particular intent repair of correct type.
     * @param sequenceMends Set of all mends of the sequence repair.
     */
    public static void generateIntentFix(
        List<Mend> sequenceMends
    ) {
        
        // Processing individual sequence mends
        for (Mend seqMend : sequenceMends) {
            
            // Adjusting edit operations from the sequence mend
            seqMend.applyRootCutting();
            
        }
        
    }
    
    
    // -------------------------------------------------------------------------

}
