
package cz.cuni.mff.corrector.repair;

import cz.cuni.mff.corrector.edit.Edit;
import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.intent.Intent;
import java.util.ArrayList;
import java.util.List;

/**
 * Abstract representation of a node repair.
 *
 * @author Martin Svoboda
 */
public abstract class NodeRepair {

    // -------------------------------------------------------------------------

    
    /**
     * Returns the sequence of repairing instructions.
     * @return List of associated repairing instructions.
     */
    public abstract List<Instruction> getInstructionsList();


    /**
     * Returns the node repair cost.
     * @return Repairing cost of the node repair.
     */
    public abstract int getRepairCost();


    // -------------------------------------------------------------------------


    /**
     * Returns the type of the node repair represented by the intent type.
     * @return Associated correction intent type.
     */
    public abstract Intent.Type getIntentType();


    // -------------------------------------------------------------------------


    /**
     * Transformes the sequence of instructions into edit operations.
     * @return Generated list of corresponding edit operations.
     */
    public List<Edit> generateNodeFix() {
        List<Edit> output = new ArrayList<Edit>();
        for (Instruction instruction : this.getInstructionsList()) {
            output.add(instruction.generateEditOperation());
        }
        return output;
    }


    // -------------------------------------------------------------------------

}
