
package cz.cuni.mff.corrector.tree;

import java.util.ArrayList;
import java.util.List;

/**
 * Representation of a data node in a data tree.
 *
 * @author Martin Svoboda
 */
public class NodeData extends Node {

    // -------------------------------------------------------------------------


    /**
     * Node data value.
     */
    private String dataValue;


    // -------------------------------------------------------------------------


    /**
     * Creates a new instance of a data node.
     * @param dataValue Textual data value.
     */
    public NodeData(String dataValue) {
        this.dataValue = dataValue;
    }


    // -------------------------------------------------------------------------


    public Type getNodeType() {
        return Type.NODE_DATA;
    }


    public String getNodeLabel() {
        return TOKEN_DT_DATA;
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the data value stored in the nodes.
     * @return Data value or <code>TOKEN_DV_UNDEFINED</code>.
     */
    public String getDataValue() {
        return this.dataValue;
    }


    /**
     * Sets a new data value of the node.
     * @param newValue Data value or <code>TOKEN_DV_UNDEFINED</code>.
     */
    public void setDataValue(String newValue) {
        this.dataValue = newValue;
    }


    public List<Node> getChildNodes() {
        return new ArrayList<Node>(0);
    }


    public int countTreeNodes() {
        return 1;
    }
    
    
    public Stats getTreeStatistics() {
        return new Stats(1, 0, 0, 1, 1);
    }
    
    
    // -------------------------------------------------------------------------


    public void serializeNode(StringBuilder builder) {
        builder.append(this.getDataValue());
    }


    public void serializeNode(StringBuilder builder, int level) {
        Node.generateIndent(builder, level);
        builder.append(this.getDataValue());
        builder.append(Node.TOKEN_SE_EOL);
    }


    // -------------------------------------------------------------------------

}
