
package cz.cuni.mff.corrector.cache;

import cz.cuni.mff.corrector.corrector.CorrectorTest;
import cz.cuni.mff.corrector.grammar.Automaton;
import cz.cuni.mff.corrector.grammar.Context;
import cz.cuni.mff.corrector.grammar.Grammar;
import cz.cuni.mff.corrector.graph.Vertex;
import cz.cuni.mff.corrector.intent.Intent;
import cz.cuni.mff.corrector.intent.IntentInsert;
import java.util.*;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * Functionality tests of caching storage.
 * 
 * @author Martin Svoboda
 */
public class CacheTest {
    
    // -------------------------------------------------------------------------


    /**
     * Tests basic functionality of caching storage.
     */
    @Test
    public void testCacheFlat() {
        
        Store<Integer> cache = new SignatureStoreFlat<Integer>();

        Grammar g = Grammar.createFromDataFile(CorrectorTest.PATH + "Grammar-1.txt");
        
        Set<String> e1 = new HashSet<String>();
        e1.add("A#1");
        e1.add("B#2");
        Context c10 = g.getStartingContext();
        Context c11 = g.getGeneralContext("A", "a");
        Context c12 = g.getGeneralContext("B", "b");
        
        List<Context> s11 = new ArrayList<Context>();
        s11.add(c11);
        List<Context> s12 = new ArrayList<Context>();
        s12.add(c11);
        s12.add(c12);
        
        IntentInsert i11 = new IntentInsert(
            new Vertex(0, Automaton.TOKEN_ST_INITIAL),
            new Vertex(1, "A#1"),
            c10, s11, "label"
        );
        IntentInsert i12 = new IntentInsert(
            new Vertex(0, Automaton.TOKEN_ST_INITIAL),
            new Vertex(1, "B#2"),
            c11, s12, "label"
        );
        
        Integer r1 = new Integer(1);
        Integer r2 = new Integer(2);
        
        cache.setStoredItem(i11, r1);
        assertSame(r1, cache.getStoredItem(i11));
        Collection<Integer> l1 = cache.getStoredItems(Intent.Type.INSERT);
        assertEquals(1, l1.size());
        assertTrue(l1.contains(r1));
        cache.setStoredItem(i12, r2);
        assertSame(r2, cache.getStoredItem(i12));
        cache.removeStoredItem(i12);
        cache.removeStoredItem(i11);
        assertNull(cache.getStoredItem(i11));
        assertNull(cache.getStoredItem(i12));
        l1 = cache.getStoredItems(Intent.Type.INSERT);
        assertEquals(0, l1.size());
        
    }
    
    
    // -------------------------------------------------------------------------
    
}
