
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.*;
import cz.cuni.mff.corrector.tree.Position;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotSame;
import org.junit.Test;

/**
 * Basic test for edit operations behaviour.
 * 
 * @author Martin Svoboda
 */
public class EditTest {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Tests basic operations with particular edit opeations.
     */
    @Test
    public void testPositionModifications() {
        
        EditAddLeaf e1 = new EditAddLeaf(new InstructionAddLeaf("label"), new Position(5));
        assertEquals("addLeaf(5, label)", e1.getOperationTitle());
        e1.applyRootPrepending(1);
        assertEquals("addLeaf(1.5, label)", e1.getOperationTitle());
        e1.applyRootShifting(2);
        assertEquals("addLeaf(3.5, label)", e1.getOperationTitle());
        e1.applyRootCutting();
        assertEquals("addLeaf(5, label)", e1.getOperationTitle());
        EditAddLeaf c1 = e1.cloneShiftedOperation(2);
        assertEquals("addLeaf(7, label)", c1.getOperationTitle());
        assertNotSame(e1, c1);
        assertNotSame(e1.getBasePosition(), c1.getBasePosition());
        
        EditAddNode e2 = new EditAddNode(new InstructionAddNode("label", 2), new Position(5));
        assertEquals("addNode(5, 2, label)", e2.getOperationTitle());
        e2.applyRootPrepending(1);
        assertEquals("addNode(1.5, 2, label)", e2.getOperationTitle());
        e2.applyRootShifting(2);
        assertEquals("addNode(3.5, 2, label)", e2.getOperationTitle());
        e2.applyRootCutting();
        assertEquals("addNode(5, 2, label)", e2.getOperationTitle());
        EditAddNode c2 = e2.cloneShiftedOperation(2);
        assertEquals("addNode(7, 2, label)", c2.getOperationTitle());
        assertNotSame(e2, c2);
        assertNotSame(e2.getBasePosition(), c2.getBasePosition());
        
        EditRemoveLeaf e3 = new EditRemoveLeaf(new InstructionRemoveLeaf(), new Position(5));
        assertEquals("removeLeaf(5)", e3.getOperationTitle());
        e3.applyRootPrepending(1);
        assertEquals("removeLeaf(1.5)", e3.getOperationTitle());
        e3.applyRootShifting(2);
        assertEquals("removeLeaf(3.5)", e3.getOperationTitle());
        e3.applyRootCutting();
        assertEquals("removeLeaf(5)", e3.getOperationTitle());
        EditRemoveLeaf c3 = e3.cloneShiftedOperation(2);
        assertEquals("removeLeaf(7)", c3.getOperationTitle());
        assertNotSame(e3, c3);
        assertNotSame(e3.getBasePosition(), c3.getBasePosition());
        
        EditRemoveNode e4 = new EditRemoveNode(new InstructionRemoveNode(), new Position(5));
        assertEquals("removeNode(5)", e4.getOperationTitle());
        e4.applyRootPrepending(1);
        assertEquals("removeNode(1.5)", e4.getOperationTitle());
        e4.applyRootShifting(2);
        assertEquals("removeNode(3.5)", e4.getOperationTitle());
        e4.applyRootCutting();
        assertEquals("removeNode(5)", e4.getOperationTitle());
        EditRemoveNode c4 = e4.cloneShiftedOperation(2);
        assertEquals("removeNode(7)", c4.getOperationTitle());
        assertNotSame(e4, c4);
        assertNotSame(e4.getBasePosition(), c4.getBasePosition());
        
        EditRenameLabel e5 = new EditRenameLabel(new InstructionRenameLabel("label"), new Position(5));
        assertEquals("renameLabel(5, label)", e5.getOperationTitle());
        e5.applyRootPrepending(1);
        assertEquals("renameLabel(1.5, label)", e5.getOperationTitle());
        e5.applyRootShifting(2);
        assertEquals("renameLabel(3.5, label)", e5.getOperationTitle());
        e5.applyRootCutting();
        assertEquals("renameLabel(5, label)", e5.getOperationTitle());
        EditRenameLabel c5 = e5.cloneShiftedOperation(2);
        assertEquals("renameLabel(7, label)", c5.getOperationTitle());
        assertNotSame(e5, c5);
        assertNotSame(e5.getBasePosition(), c5.getBasePosition());
        
    }
    
    
    // -------------------------------------------------------------------------
    
}
