
package cz.cuni.mff.corrector.repair;

import cz.cuni.mff.corrector.edit.*;
import cz.cuni.mff.corrector.graph.Graph;
import cz.cuni.mff.corrector.graph.GraphTest;
import cz.cuni.mff.corrector.instruction.InstructionAddLeaf;
import cz.cuni.mff.corrector.instruction.InstructionRemoveLeaf;
import cz.cuni.mff.corrector.instruction.InstructionRemoveNode;
import cz.cuni.mff.corrector.instruction.InstructionRenameLabel;
import cz.cuni.mff.corrector.intent.Intent;
import cz.cuni.mff.corrector.tree.Position;
import java.util.ArrayList;
import java.util.List;
import static org.junit.Assert.assertEquals;
import org.junit.Test;

/**
 * Tests for sequence repairs.
 * 
 * @author Martin Svoboda
 */
public class SequenceRepairTest {
    
    // -------------------------------------------------------------------------


    /**
     * Relative path from the home folder to testing XML documents.
     */
    public static String PATH = "test/cz/cuni/mff/corrector/data/";
    
    
    // -------------------------------------------------------------------------


    /**
     * Tests the method for generating the product of mend lists.
     */
    @Test
    public void testCachingCorrector() {
        
        Graph g = GraphTest.generateSampleGraph();
        
        SequenceRepair r = new SequenceRepair(g, Intent.Type.CORRECT);
        
        List<List<Mend>> l = new ArrayList<List<Mend>>();
        List<Mend> s;
        
        // Generating the first component
        List<Edit> es11 = new ArrayList<Edit>();
        es11.add(new EditAddLeaf(new InstructionAddLeaf("lab1"), new Position(3)));
        Mend m11 = new Mend(es11, 2);
        List<Edit> es12 = new ArrayList<Edit>();
        es12.add(new EditAddLeaf(new InstructionAddLeaf("lab2"), new Position(5)));
        Mend m12 = new Mend(es12, 4);
        s = new ArrayList<Mend>();
        l.add(s);
        s.add(m11);
        s.add(m12);
        
        // Generating the second component
        List<Edit> es21 = new ArrayList<Edit>();
        List<Integer> pi211 = new ArrayList<Integer>();
        pi211.add(4); pi211.add(1); pi211.add(0);
        es21.add(new EditRemoveLeaf(new InstructionRemoveLeaf(), new Position(pi211)));
        List<Integer> pi212 = new ArrayList<Integer>();
        pi212.add(4); pi212.add(1); pi212.add(0);
        es21.add(new EditRemoveLeaf(new InstructionRemoveLeaf(), new Position(pi212)));
        List<Integer> pi213 = new ArrayList<Integer>();
        pi213.add(4); pi213.add(1);
        es21.add(new EditRemoveLeaf(new InstructionRemoveLeaf(), new Position(pi213)));
        Mend m21 = new Mend(es21, 2);
        s = new ArrayList<Mend>();
        l.add(s);
        s.add(m21);
        
        // Generating the third component
        List<Edit> es31 = new ArrayList<Edit>();
        es31.add(new EditRenameLabel(new InstructionRenameLabel("lab3"), new Position(8)));
        Mend m31 = new Mend(es31, 2);
        List<Edit> es32 = new ArrayList<Edit>();
        es32.add(new EditRemoveNode(new InstructionRemoveNode(), new Position(8)));
        Mend m32 = new Mend(es32, 2);
        s = new ArrayList<Mend>();
        l.add(s);
        s.add(m31);
        s.add(m32);
        
        // Computing lists combination
        List<Mend> o = new ArrayList<Mend>();
        r.combineNestedMends(o, l);
        o = new ArrayList<Mend>();
        r.combineNestedMends(o, l);
        assertEquals(4, o.size());
        
        // Printing returned results
        for (Mend m : o) {
            System.out.print("mend:\n");
            System.out.print(m.getMendSerialization("  ", "\n"));
        }
        
        // Testing result correctness
        Mend m1 = o.get(0);
        List<Edit> e1 = m1.getEditSequence();
        assertEquals("addLeaf(3, lab1)", e1.get(0).getOperationTitle());
        assertEquals("removeLeaf(6.1.0)", e1.get(1).getOperationTitle());
        assertEquals("removeLeaf(6.1.0)", e1.get(2).getOperationTitle());
        assertEquals("removeLeaf(6.1)", e1.get(3).getOperationTitle());
        assertEquals("renameLabel(12, lab3)", e1.get(4).getOperationTitle());
        Mend m2 = o.get(1);
        List<Edit> e2 = m2.getEditSequence();
        assertEquals("addLeaf(3, lab1)", e2.get(0).getOperationTitle());
        assertEquals("removeLeaf(6.1.0)", e2.get(1).getOperationTitle());
        assertEquals("removeLeaf(6.1.0)", e2.get(2).getOperationTitle());
        assertEquals("removeLeaf(6.1)", e2.get(3).getOperationTitle());
        assertEquals("removeNode(12)", e2.get(4).getOperationTitle());
        Mend m3 = o.get(2);
        List<Edit> e3 = m3.getEditSequence();
        assertEquals("addLeaf(5, lab2)", e3.get(0).getOperationTitle());
        assertEquals("removeLeaf(8.1.0)", e3.get(1).getOperationTitle());
        assertEquals("removeLeaf(8.1.0)", e3.get(2).getOperationTitle());
        assertEquals("removeLeaf(8.1)", e3.get(3).getOperationTitle());
        assertEquals("renameLabel(14, lab3)", e3.get(4).getOperationTitle());
        Mend m4 = o.get(3);
        List<Edit> e4 = m4.getEditSequence();
        assertEquals("addLeaf(5, lab2)", e4.get(0).getOperationTitle());
        assertEquals("removeLeaf(8.1.0)", e4.get(1).getOperationTitle());
        assertEquals("removeLeaf(8.1.0)", e4.get(2).getOperationTitle());
        assertEquals("removeLeaf(8.1)", e4.get(3).getOperationTitle());
        assertEquals("removeNode(14)", e4.get(4).getOperationTitle());

    }
    
    
    // -------------------------------------------------------------------------
    
}
