
package cz.cuni.mff.corrector.tree;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Testing class for data trees.
 *
 * @author Martin Svoboda
 */
public class TreeTest {

    // -------------------------------------------------------------------------


    /**
     * XML prologue string.
     */
    public static String PROLOGUE =
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";

    /**
     * Relative path from the home folder to testing XML documents.
     */
    public static String PATH = "test/cz/cuni/mff/corrector/tree/";

    
    /**
     * Number of prepared testing XML documents pairs.
     */
    public static int FILES = 3;

    
    // -------------------------------------------------------------------------


    /**
     * Tests correct construction of data trees from XML documents.
     */
    @Test
    public void testTreeConstruction() {

        // Testing correctness of trees with basic serialization
        for (int i = 1; i <= FILES; i++) {
            // Loading input XML document
            String fileName = PATH + "Tree-1-" + i + ".xml";
            String input = readInputFile(fileName);
            // Creating data tree sructure
            Tree tree = Tree.createFromXmlFile(fileName);
            assertNotNull(tree);
            // Serializing data tree
            String output = tree.serializeDataTree(PROLOGUE, false);
            assertEquals(input, output);
        }

        // Testing correctness of trees with indented serialization
        for (int i = 1; i <= FILES; i++) {
            // Loading input XML document
            String fileName = PATH + "Tree-2-" + i + ".xml";
            String input = readInputFile(fileName);
            // Creating data tree sructure
            Tree tree = Tree.createFromXmlFile(fileName);
            assertNotNull(tree);
            // Serializing data tree
            String output = tree.serializeDataTree(PROLOGUE, true);
            assertEquals(input, output);
        }

    }


    // -------------------------------------------------------------------------


    /**
     * Reads entire input file and stores its content into a string.
     * @param fileName Path to a file to be load.
     * @return String with the content of the specified file.
     */
    private String readInputFile(String fileName) {
        try {
            FileReader fileReader = new FileReader(fileName);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            StringBuilder stringBuilder = new StringBuilder();
            String stringLine;
            while ((stringLine = bufferedReader.readLine()) != null) {
                stringBuilder.append(stringLine).append("\n");
            }
            return stringBuilder.toString();
        } catch (FileNotFoundException exception) {
            return null;
        } catch (IOException exception) {
            return null;
        }
    }


    // -------------------------------------------------------------------------

}
